"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startFoundationSimulationServer = exports.createFoundationSimulationServer = void 0;
const express_1 = __importDefault(require("express"));
const lodash_1 = require("lodash");
const openapi_backend_1 = __importDefault(require("openapi-backend"));
const store_1 = require("./store");
async function createFoundationSimulationServer({ port = 9000, openapi, extendStore, extendRouter, }) {
    console.time(`api listening at http://localhost:${port}, started in`);
    let app = (0, express_1.default)();
    app.use(express_1.default.json());
    let simulationStore = (0, store_1.createSimulationStore)(extendStore);
    if (extendRouter) {
        extendRouter(app, simulationStore);
    }
    if (openapi) {
        let { document, handlers, apiRoot } = openapi;
        let mergedOAS = Array.isArray(document)
            ? mergeDocumentArray(document)
            : document;
        let api = new openapi_backend_1.default({ definition: mergedOAS, apiRoot });
        // register your framework specific request handlers here
        let handlerObjectRegistration = (handlerEntries, prefix) => {
            for (let [key, handler] of Object.entries(handlerEntries)) {
                if (typeof handler === "object") {
                    handlerObjectRegistration(handler, key);
                }
                else {
                    api.register(`${prefix ? `${prefix}/` : ""}${key}`, handler);
                }
            }
        };
        handlerObjectRegistration(handlers(simulationStore));
        api.register({
            validationFail: (c, req, res) => res.status(400).json({ err: c.validation.errors }),
            notFound: (c, req, res) => res.status(404).json({ error: "not found" }),
            notImplemented: (c, req, res) => {
                let { status, mock } = c.api.mockResponseForOperation(
                // the route validates this exists and throws if it does not
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                c.operation.operationId);
                return res.status(status).json(mock);
            },
        });
        // initalize the backend
        api.init();
        app.use((req, res) => api.handleRequest(req, req, res));
    }
    return {
        listen: async (portOverride) => {
            let listeningPort = portOverride !== null && portOverride !== void 0 ? portOverride : port;
            let server = app.listen(listeningPort, () => console.timeEnd(`api listening at http://localhost:${listeningPort}, started in`));
            if (!server.listening) {
                await new Promise((resolve) => {
                    server.once("listening", resolve);
                });
            }
            return server;
        },
    };
}
exports.createFoundationSimulationServer = createFoundationSimulationServer;
const mergeDocumentArray = (documents) => {
    let [firstDocument, secondDocument, ...remainingDocuments] = documents;
    let document = (0, lodash_1.merge)({ ...firstDocument }, { ...secondDocument });
    if (remainingDocuments.length > 0)
        return mergeDocumentArray([document].concat(remainingDocuments));
    return document;
};
async function startFoundationSimulationServer(arg) {
    let simulation = await createFoundationSimulationServer(arg);
    return simulation.listen();
}
exports.startFoundationSimulationServer = startFoundationSimulationServer;
//# sourceMappingURL=index.js.map